%%%%%%%%%%%%%%%%%%  Generate Fig. 2 in
%%% "Optimized Sparse Projections for Compressive Sensing,"
%%%  https://arxiv.org/abs/1709.06895.  
%%%  by T. Hong, X. Li, Z. Zhu, and Qiuwei Li

clear all; close all;
addpath('tools');
load('Psi_big.mat')               %load the trained Dictionary and the corresponding Coeffcient matrix,which is obtained via K-SVD algorithm using the training data

%read the image====================================================
pathForImages ='images/';
imageName = 'lena512.png';
image = imread(strcat([pathForImages,imageName]));
image = double(image);

image_or = image;    
blocksize = 16;
% [image,mblk_I,nblk_I] = our_im2col(image,blocksize);  % rearrange image 
% imageMeans = mean(image);
% image = image-ones(size(image,1),1)*imageMeans; %remove the means


%param setting=============================================================
K = 16;                              %the sparsity
M = 80;                             %the dimension of sensing matrix
N = blocksize^2;
setpsize = 10;                    %the stepsize in varyAlpha
varysparsity = [10:setpsize:256];       %the varying alpha
DataNum = 1;
%==========================================================================

%test the differenet kinds of designing sensing matrix
%CS_randn =============================================================
randn('state',1)
Phi_randn = randn(M,N); %the optimal sensing matrix used in CS_randn
[ X_randn,PSNR_randn] = image_recons( Phi_randn, Psi, image,K,blocksize  );
PSNR_randn = ones(1,length(varysparsity)) *PSNR_randn;
%======================================================================

%test CS_bispar using different sparsity
DataNum = 5;
for I=1:length(varysparsity)    
    k = varysparsity(I);
    for iter = 1:DataNum
        Phi_sparse_binary = zeros(M,N);
        for II = 1:M
            randnposi= randperm(N);
            Phi_sparse_binary(II,randnposi(1 : ceil(k))) = randn(1,ceil(k));
        end
        [ X_sparse_binary,PSNR_sparse_binary(iter,I)] = image_recons( Phi_sparse_binary, Psi, image,K,blocksize  );
    end
    %dispaly the progress
    display(['CS_bispar, the progress is i = ', num2str(I), '/',num2str(length(varysparsity))]);
end
if DataNum>1
    PSNR_sparse_binary = mean(PSNR_sparse_binary);
end
DataNum = 1;

%test CS_sparse_IHTA using different sparsity
for I=1:length(varysparsity)    
    k = varysparsity(I);
    for iter = 1:DataNum
        [Phi_sparse_IHTA] = SparseSensing( dctmtx(N),Psi,k,0.5,M,1);
        [ X_sparse_IHTA,PSNR_sparse_IHTA(iter,I) ] = image_recons( Phi_sparse_IHTA, Psi, image,K,blocksize  );
    end
    %dispaly the progress
    display(['CS_sparse_IHTA, the progress is i = ', num2str(I), '/',num2str(length(varysparsity))]);
end
if DataNum>1
    PSNR_sparse_IHTA = mean( PSNR_sparse_IHTA);
end

%test CS_sparse_IHT using different sparsity
for I=1:length(varysparsity)    
    k = varysparsity(I);
    for iter = 1:DataNum
        [Phi_sparse_IHT] = SparseSensing( eye(N),Psi,k,0.5,M,1);
        [ X_sparse_IHT,PSNR_sparse_IHT(iter,I) ] = image_recons( Phi_sparse_IHT, Psi, image,K,blocksize  );
    end
    %dispaly the progress
    display(['CS_sparse_IHT, the progress is i = ', num2str(I), '/',num2str(length(varysparsity))]);
end
if DataNum>1
    PSNR_sparse_IHT = mean( PSNR_sparse_IHT);
end

%% plot the results
figure(1)
h1=plot(varysparsity(1):setpsize:varysparsity(end) ,PSNR_randn,'k-','linewidth',2.2);
set(gcf,'Position',[100 300 800 300]);
set(gca,'FontSize',10,'XTick', [10 50 100 150 200 256]);
xlabel('$\kappa$','fontsize',16,'FontName','Times New Roman','Interpreter','LaTex');
h1=ylabel('\sigma_{\it {psnr}}');
set(h1,'FontSize',14);
axis([0 256   30 35]);
hold on
plot(varysparsity(1):setpsize:varysparsity(end),PSNR_sparse_IHT(end)*ones(1,length(varysparsity)+1),'k--','linewidth',2.2);
plot(varysparsity(1):setpsize:varysparsity(end),PSNR_sparse_binary,'k-*','linewidth',2.2);
plot(varysparsity(1):setpsize:varysparsity(end),PSNR_sparse_IHTA,'b-o','linewidth',2.2);
plot(varysparsity(1):setpsize:varysparsity(end),PSNR_sparse_IHT,'r-d','linewidth',2.2);
legend('CS_{\it{randn}}','CS_{\it{MT}}','CS_{\it{bispar}}','CS_{\it{sparse-A}}','CS_{\it{sparse}}');